# SDRAM controller for the Colorlight 5a-75b


from amaranth import Module, Cat, Signal, Assert, unsigned
from amaranth.build import Platform
from amaranth.lib import wiring, data, enum
from amaranth.lib.wiring import In, Out


# RAM Specs:
# 4 banks, 2048 rows, 256 columns.
# 11 row bits, 8 column bits.
# word size = 32
# 8 megabytes data.


class SDRAMSignature(wiring.Signature):
    """Signature of a variable-size sdram. Data is split between in/out and has out_en"""

    def __init__(self, addr_width, data_width=32, bank_width=2):
        super().__init__(
            {
                "nCS": Out(1),
                "cke": Out(1),
                "nRAS": Out(1),
                "nCAS": Out(1),
                "nWE": Out(1),
                "addr": Out(addr_width),
                "data_out": Out(data_width),
                "data_in": In(data_width),
                # todo: use dqm
                "data_wren": Out(1),
                "bank_cs": Out(bank_width),
            }
        )


class _WriteBurstLength(enum.Enum, shape=1):
    """MRS Write burst mode"""

    BURST = 0
    SINGLE_BIT = 1


class _TestMode(enum.Enum, shape=2):
    """The "test mode" of the sdram. This is always zero pretty much"""

    MODE_REGISTER_SET = 0
    RESERVED0 = 1
    RESERVED1 = 2
    RESERVED2 = 3


class _CASLatency(enum.Enum, shape=3):
    """How many cycles of latency for the column address select to complete"""

    CYCL2 = 2
    CYCL3 = 3


class _BurstType(enum.Enum, shape=1):
    SEQUENTIAL = 0
    INTERLEAVED = 1


class _BurstLength(enum.IntEnum, shape=3):
    """The size of the burst"""

    SINGLE = 0
    DUAL = 1
    QUAD = 2
    OCT = 3
    FULL_PAGE = 7  # this is 256 words?


class _Command(enum.Enum):
    """Command set for SDRAM"""

    MRS_WRITE = 0
    ACTIVATE = 1
    PRECHARGE = 2
    WRITE = 3
    READ = 4
    CBR = 5  # auto refresh
    SELF_REFRESH = 6
    BRST_STOP = 7
    NOP = 8


class BankController(wiring.Component):
    """Manages a single Bank. Has a bank locking/state tracker,
    can issue commands"""
